# -*- coding: utf-8 -*-
"""
REST API serialization classes.
"""

from __future__ import absolute_import, unicode_literals

from rest_framework import serializers

from .models import UserTaskArtifact, UserTaskStatus


class StatusSerializer(serializers.HyperlinkedModelSerializer):
    """
    REST API serializer for the UserTaskStatus model.
    """

    artifacts = serializers.HyperlinkedRelatedField(many=True, read_only=True, view_name='usertaskartifact-detail',
                                                    lookup_field='uuid')

    class Meta:
        """
        Status serializer settings.
        """

        model = UserTaskStatus
        fields = (
            'name', 'state', 'state_text', 'completed_steps', 'total_steps', 'attempts', 'created', 'modified',
            'artifacts'
        )


class ArtifactSerializer(serializers.HyperlinkedModelSerializer):
    """
    REST API serializer for the UserTaskArtifact model.
    """

    file = serializers.SerializerMethodField()

    class Meta:
        """
        Artifact serializer settings.
        """

        model = UserTaskArtifact
        fields = ('name', 'created', 'modified', 'status', 'file', 'text', 'url')
        extra_kwargs = {
            'status': {'lookup_field': 'uuid'},
        }

    def get_file(self, obj):
        """
        Get the URL of the artifact's associated file data.

        Arguments:
            obj (UserTaskArtifact): The artifact being serialized

        Returns
        -------
            six.text_type: The URL of the artifact's file field (empty if there isn't one)

        """
        if not obj.file:
            return ''
        return obj.file.url



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-07 20:22
from __future__ import unicode_literals

import uuid

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserTaskArtifact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, help_text='Unique ID for use in APIs', unique=True)),
                ('name', models.CharField(default='Output', help_text='Distinguishes between multiple artifact types for the same task', max_length=255)),
                ('file', models.FileField(blank=True, null=True, upload_to='user_tasks/%Y/%m/%d/')),
                ('url', models.URLField(blank=True)),
                ('text', models.TextField(blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='UserTaskStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, help_text='Unique ID for use in APIs', unique=True)),
                ('task_id', models.CharField(db_index=True, help_text='UUID of the associated Celery task', max_length=128, unique=True)),
                ('is_container', models.BooleanField(default=False, help_text='True if this status corresponds to a container of multiple tasks')),
                ('task_class', models.CharField(help_text='Fully qualified class name of the task being performed', max_length=128)),
                ('name', models.CharField(help_text='A name for this task which the triggering user will understand', max_length=255)),
                ('state', models.CharField(default='Pending', max_length=128)),
                ('completed_steps', models.PositiveSmallIntegerField(default=0)),
                ('total_steps', models.PositiveSmallIntegerField()),
                ('attempts', models.PositiveSmallIntegerField(default=1, help_text='How many times has execution been attempted?')),
                ('parent', models.ForeignKey(blank=True, default=None, help_text='Status of the containing task grouping (if any)', null=True, on_delete=django.db.models.deletion.CASCADE, to='user_tasks.UserTaskStatus')),
                ('user', models.ForeignKey(help_text='The user who triggered the task', on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'user task statuses',
            },
        ),
        migrations.AddField(
            model_name='usertaskartifact',
            name='status',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='artifacts', to='user_tasks.UserTaskStatus'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Django admin configuration for the ``django-user-tasks`` models.
"""

from __future__ import absolute_import, unicode_literals

from django.contrib import admin

from .models import UserTaskArtifact, UserTaskStatus

admin.site.register(UserTaskArtifact)
admin.site.register(UserTaskStatus)



"""
Management of user-triggered asynchronous tasks in Django projects.
"""

from __future__ import absolute_import, unicode_literals

from django.dispatch import Signal

__version__ = '0.1.5'

default_app_config = 'user_tasks.apps.UserTasksConfig'  # pylint: disable=invalid-name

# This signal is emitted when a user task reaches any final state:
# SUCCEEDED, FAILED, or CANCELED
user_task_stopped = Signal(providing_args=['status'])  # pylint: disable=invalid-name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Sample implementation of authorization rules for the ``django-user-tasks`` permissions.

Requires use of the ``rules`` package.  The rule implementations aren't
registered by default because that would interfere with the ability to use
custom rules.)  You can register them in another application's ``rules.py``:

.. code-block:: python

    import user_tasks

    user_tasks.rules.add_rules()
"""

from __future__ import absolute_import, unicode_literals

import rules


@rules.predicate
def is_status_creator(user, status=None):
    """
    Check for ownership of the provided status.

    Arguments:
        user (User): The user for whom a permission check is being made.
        status (UserTaskStatus): The status record to check ownership for.

    Returns
    -------
        bool: True if no status is provided or the user owns the given status.

    """
    if not status:
        return True
    return status.user_id == user.id


@rules.predicate
def is_artifact_creator(user, artifact=None):
    """
    Check for ownership of the provided artifact.

    Arguments:
        user (User): The user for whom a permission check is being made.
        artifact (UserTaskArtifact): The artifact to check ownership for.

    Returns
    -------
        bool: True if no artifact is provided or the user owns the given artifact.

    """
    if not artifact:
        return True
    return is_status_creator(user, artifact.status)


STATUS_PERMISSION = is_status_creator | rules.predicates.is_superuser
ARTIFACT_PERMISSION = is_artifact_creator | rules.predicates.is_superuser


def add_rules():
    """
    Use the rules provided in this module to implement authorization checks for the ``django-user-tasks`` models.

    These rules allow only superusers and the user who triggered a task to view its status or artifacts, cancel the
    task, or delete the status information and all its related artifacts.  Only superusers are allowed to directly
    modify or delete an artifact (or to modify a task status record).
    """
    rules.add_perm('user_tasks.view_usertaskstatus', STATUS_PERMISSION)
    rules.add_perm('user_tasks.cancel_usertaskstatus', STATUS_PERMISSION)
    rules.add_perm('user_tasks.change_usertaskstatus', rules.predicates.is_superuser)
    rules.add_perm('user_tasks.delete_usertaskstatus', STATUS_PERMISSION)
    rules.add_perm('user_tasks.view_usertaskartifact', ARTIFACT_PERMISSION)
    rules.add_perm('user_tasks.change_usertaskartifact', rules.predicates.is_superuser)
    rules.add_perm('user_tasks.delete_usertaskartifact', rules.predicates.is_superuser)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Optional Django REST Framework filter backends for the ``django-user-tasks`` REST API.
"""

from __future__ import absolute_import, unicode_literals

from rest_framework.filters import BaseFilterBackend


class ArtifactFilterBackend(BaseFilterBackend):
    """
    Default filter for UserTaskArtifact listings in the REST API.

    Ensures that superusers can see all artifacts, but other users
    can only see artifacts for tasks they personally triggered.
    """

    def filter_queryset(self, request, queryset, view):
        """
        Filter out any artifacts which the requesting user does not have permission to view.
        """
        if request.user.is_superuser:
            return queryset
        return queryset.filter(status__user=request.user)


class StatusFilterBackend(BaseFilterBackend):
    """
    Default filter for UserTaskStatus listings in the REST API.

    Ensures that superusers can see all task status records, but other users
    can only see records for tasks they personally triggered.
    """

    def filter_queryset(self, request, queryset, view):
        """
        Filter out any status records which the requesting user does not have permission to view.
        """
        if request.user.is_superuser:
            return queryset
        return queryset.filter(user=request.user)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-12-05 16:12
from __future__ import unicode_literals

from django.db import migrations, models

from user_tasks.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('user_tasks', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='usertaskartifact',
            name='file',
            field=models.FileField(
                blank=True, null=True, storage=settings.USER_TASKS_ARTIFACT_STORAGE, upload_to='user_tasks/%Y/%m/%d/'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable=C0111,W6005,W6100
from __future__ import absolute_import, print_function

import os
import re
import sys

from setuptools import setup


def get_version(*file_paths):
    """
    Extract the version string from the file at the given relative path fragments.
    """
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')

VERSION = get_version('user_tasks', '__init__.py')

if sys.argv[-1] == 'tag':
    print("Tagging the version on github:")
    os.system("git tag -a %s -m 'version %s'" % (VERSION, VERSION))
    os.system("git push --tags")
    sys.exit()

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
CHANGELOG = open(os.path.join(os.path.dirname(__file__), 'CHANGELOG.rst')).read()

setup(
    name='django-user-tasks',
    version=VERSION,
    description="""Management of user-triggered asynchronous tasks in Django projects""",
    long_description=README + '\n\n' + CHANGELOG,
    author='edX',
    author_email='oscm@edx.org',
    url='https://github.com/edx/django-user-tasks',
    packages=[
        'user_tasks',
    ],
    include_package_data=True,
    install_requires=['django-model-utils>=2.0', 'djangorestframework>=3.0.4'],
    license="Apache Software License 2.0",
    zip_safe=False,
    keywords='Django edx',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
)



# -*- coding: utf-8 -*-
"""
REST API endpoints.
"""

from __future__ import absolute_import, unicode_literals

from rest_framework import mixins, permissions, viewsets
from rest_framework.decorators import detail_route
from rest_framework.response import Response

from .conf import settings
from .models import UserTaskArtifact, UserTaskStatus
from .serializers import ArtifactSerializer, StatusSerializer


class DjangoObjectPermissionsIncludingView(permissions.DjangoObjectPermissions):
    """
    Django REST Framework object permissions including ``<app>.view_<model>``.

    Yields better response codes than using the superclass directly, assuming
    you've defined a view_* permission for the models.  The rationale for
    overriding perms_map is described in the Django REST framework
    documentation for `permissions`_ and `filters`_.

    .. _permissions: http://www.django-rest-framework.org/api-guide/permissions/#djangoobjectpermissions
    .. _filters: http://www.django-rest-framework.org/api-guide/filtering/#djangoobjectpermissionsfilter
    """

    perms_map = {
        'GET': ['%(app_label)s.view_%(model_name)s'],
        'OPTIONS': ['%(app_label)s.view_%(model_name)s'],
        'HEAD': ['%(app_label)s.view_%(model_name)s'],
        'POST': ['%(app_label)s.cancel_%(model_name)s'],
        'PUT': ['%(app_label)s.change_%(model_name)s'],
        'PATCH': ['%(app_label)s.change_%(model_name)s'],
        'DELETE': ['%(app_label)s.delete_%(model_name)s'],
    }


class StatusViewSet(
        mixins.DestroyModelMixin,
        mixins.ListModelMixin,
        mixins.RetrieveModelMixin,
        viewsets.GenericViewSet
):
    """
    REST API endpoints for user-triggered asynchronous tasks.

    The status of a task can be viewed, listed, deleted, or used to cancel the
    underlying task.
    """

    filter_backends = settings.USER_TASKS_STATUS_FILTERS
    lookup_field = 'uuid'
    permission_classes = (DjangoObjectPermissionsIncludingView,)
    queryset = UserTaskStatus.objects.order_by('-created')
    serializer_class = StatusSerializer

    @detail_route(methods=['post'])
    def cancel(self, request, *args, **kwargs):  # pylint: disable=unused-argument
        """
        Cancel the task associated with the specified status record.

        Arguments:
            request (Request): A POST including a task status record ID

        Returns
        -------
            Response: A JSON response indicating whether the cancellation succeeded or not

        """
        status = self.get_object()
        status.cancel()
        serializer = StatusSerializer(status, context={'request': request})
        return Response(serializer.data)


class ArtifactViewSet(viewsets.ReadOnlyModelViewSet):
    """
    REST API endpoints for asynchronous task artifacts.

    Artifact data can only be viewed or listed, not modified in any way via
    this API.
    """

    filter_backends = settings.USER_TASKS_ARTIFACT_FILTERS
    lookup_field = 'uuid'
    permission_classes = (DjangoObjectPermissionsIncludingView,)
    queryset = UserTaskArtifact.objects.all()
    serializer_class = ArtifactSerializer
